//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#include "ReadTriggers.h"

using namespace std;

int main (int argc, char* argv[]){

  // check the argument
  if(argc!=3){
    cerr<<argv[0]<<"usage:"<<endl; 
    cerr<<argv[0]<<" [input root file] [segment file]"<<endl; 
    cerr<<endl; 
    cerr<<"[input root file]:   input trigger file to split"<<endl; 
    cerr<<"[segment file]:      list of time segments to keep"<<endl; 
    cerr<<endl; 
    return 1;
  }

  // input files
  string infile = (string)argv[1];
 
  // input segments
  string inseg = (string)argv[2];
  Segments *InSeg = new Segments(inseg);
  if(InSeg->GetLiveTime()<=0){
    cerr<<"---> in segments: no livetime!"<<endl;
    delete InSeg;
    return 0;
  }
 
  // extract trigger segments
  cout<<"extracting metadata..."<<endl;
  ReadTriggerMetaData *meta = new ReadTriggerMetaData(infile,"",0);
  if(meta->GetSegments()->GetLiveTime()<=0){
    cerr<<"---> in triggers: no livetime!"<<endl;
    delete InSeg;
    delete meta;
    return 0;
  }

  // output segments
  Segments *OutSeg = new Segments(inseg);
  OutSeg->Intersect(meta->GetSegments());
  if(OutSeg->GetLiveTime()<=0){
    cerr<<"---> out segment: no livetime!"<<endl;
    delete InSeg;
    delete meta;
    delete OutSeg;
    return 0;
  }
  if(OutSeg->GetLiveTime()==meta->GetSegments()->GetLiveTime()){
    cerr<<"---> in/out: same livetime!"<<endl;
    delete InSeg;
    delete meta;
    delete OutSeg;
    return 0;
  }

  // stream
  Streams *St = new Streams(meta->GetStreamName(),0);

  // output file name (GWOLLUM convention)
  cout<<"output file name:"<<endl;
  int start=(int)floor(OutSeg->GetStart(0));
  int duration=(int)ceil(OutSeg->GetEnd(OutSeg->GetNsegments()-1))-start;
  ostringstream outfilestream;
  outfilestream<<"./"<<St->GetNameConv()<<"_"<<meta->GetProcessName()<<"-"<<start<<"-"<<duration<<".root";
  string outfilename = outfilestream.str();
  cout<<"---> "<<outfilename<<endl;
  delete St;
  delete meta;

  // input file
  TFile InFile(infile.c_str());

  // get trigger tree 
  TTree *inTtree;
  InFile.GetObject("triggers", inTtree);
  const int nentriesT = inTtree->GetEntries();
  double tstart=0.0;
  inTtree->SetBranchAddress("tstart", &tstart);

  // get metadata tree 
  TTree *inMtree;
  InFile.GetObject("metadata", inMtree);
  const int nentriesM = inMtree->GetEntries();
  double Mtstart=0.0;
  double Mtend=0.0;
  inMtree->SetBranchAddress("start", &Mtstart);
  inMtree->SetBranchAddress("end", &Mtend);

  // output file
  TFile OutFile(outfilename.c_str(),"RECREATE");

  // output trigger tree
  auto outTtree = inTtree->CloneTree(0);
  for(int t=0; t<nentriesT; t++) {
    inTtree->GetEntry(t);
    if (OutSeg->IsInsideSegment(tstart))
      outTtree->Fill();
  }
  outTtree->Print();

  // output metadata tree
  int seg_n, seg_n_prev=-1;
  auto outMtree = inMtree->CloneTree(0);
  for(int m=0; m<nentriesM; m++) {
    inMtree->GetEntry(m);
    if (OutSeg->IsInsideSegment(Mtstart,seg_n)){
      if(seg_n==seg_n_prev) continue;
      Mtstart=OutSeg->GetStart(seg_n);
      Mtend=OutSeg->GetEnd(seg_n);
      seg_n_prev=seg_n;
      outMtree->Fill();
    }
  }
  outMtree->Print();

  // output segment tree
  TTree *outStree = OutSeg->GetTree();
  outStree->Print();

  OutFile.Write();
  OutFile.Close();  
  return 0;
}

