//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#ifndef __CUtils__
#define __CUtils__

#include <iostream>
#include <iomanip>
#include <math.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <vector>
#include <stdio.h>
#include <sstream>
//#include <strstream>
#include <fstream>
#include <regex.h>
// FIXME: this is a bug for ROOT v < 5.26 -> to be removed later
#ifndef __CINT__
#include <dirent.h>
#include <glob.h>
#include <fnmatch.h>
#endif

#define LN2 0.6931471805599453094172321214581766  /**< log_e 2 */
#define LN10 2.30258509299404590109361379290930926799774169921875  /**< log(10) */
#define SQRT2 1.4142135623730951454746218587388284504413604736328125
using namespace std;

/**
 * Is this integer a power of 2.
 * true is returned if this is the case.
 * @param x integer to test.
 */
inline bool IsPowerOfTwo (unsigned int x){ return ((x != 0) && !(x & (x - 1))); };


/**
 * Returns the next power of 2.
 * @param x to get the next power of 2 of
 */
inline int NextPowerOfTwo (double x){ 
  return (int)pow(2.0,ceil(log(x)/log(2.0))); 
};


/**
 * Returns the bias factor when computing a median with nn samples.
 * @param nn number of samples
 */
double medianbiasfactor(const int nn);

/**
 * Splits a string.
 * A string is divided into an array of words. The separator defines where to break the string. The separator should be a single character.
 * A vector of string is returned.
 * @param stringtodivide string to divide
 * @param separator separator
 */
vector<string> SplitString(const string stringtodivide, const char separator);

/**
 * Convert a string to uppercase.
 * @param stringtoconvert string to convert
 */
string StringToUpper(string stringtoconvert);

/**
 * Returns the file name given the file path.
 * For example "myprog.exe" is returned from:
 * - the full path "/usr/bin/myprog.exe"
 * - the relative path "./bin/myprog.exe"
 * @param filepathname file path
 */
inline string GetFileNameFromPath(const string filepathname){ return filepathname.substr(filepathname.find_last_of("/")+1); };

/**
 * Extract and return the file name from file path.
 * See GetFileNameFromPath().
 * @param filepath file path
 */
inline string GetFileName(const string filepath){ return GetFileNameFromPath(filepath); };

/**
 * Replace all occurences of a sub-string.
 * All occurences of substring 'from' are replaced by 'to'.
 * The modified string is returned.
 * @param str string to modify
 * @param from substring to replace from
 * @param from substring to replace to
 */
string ReplaceAll(string str, const string& from, const string& to);

/**
 * Check if a directory exists.
 * Returns true if 'dirname' is an existing directory.
 * @param dirname path to directory to test
 */
bool IsDirectory(const char* dirname);

/**
 * Check if a directory exists.
 * Returns true if 'dirname' is an existing directory.
 * @param dirname path to directory to test
 */
inline bool IsDirectory(const string dirname){ return IsDirectory(dirname.c_str()); };

/**
 * Returns list of subdirectories.
 * The vector 'subdir' is filled with directories contained in 'maindir'
 * Returns true if success
 * @param maindir path to main directory
 */
bool ListDirectories(const string maindir, vector <string> &subdir);

/**
 * Check if a file exists and is a txt file.
 * Returns true if 'filename' is an existing text file.
 * @param filename path to file to test
 */
bool IsTextFile(const char* filename);

/**
 * Check if a file exists and is a txt file.
 * Returns true if 'filename' is an existing text file.
 * @param filename path to file to test
 */
inline bool IsTextFile(const string filename){ return IsTextFile(filename.c_str()); };

/**
 * Check if a file exists and is a binary file.
 * Returns true if 'filename' is an existing binary file.
 * @param filename path to file to test
 */
bool IsBinaryFile(const char* filename);

/**
 * Check if a file exists and is a binary file.
 * Returns true if 'filename' is an existing binary file.
 * @param filename path to file to test
 */
inline bool IsBinaryFile(const string filename){ return IsBinaryFile(filename.c_str()); };

/**
 * c utility to extract a list of files.
 * Returns a list of files matching a pattern.
 * @param pattern file pattern
 */
vector<string> Glob(const char* pattern);

#endif


